# Generated by Django 2.2.2 on 2019-06-20 12:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_htcpcp_tea', '0002_auto_20190619_1041'),
    ]

    operations = [
        migrations.AlterField(
            model_name='teatype',
            name='name',
            field=models.CharField(db_index=True, max_length=35, unique=True, verbose_name='Tea name'),
        ),
    ]



#  Copyright (c) 2019 Brian Schubert
#
#  This file is distributed under the MIT License. If a copy of the
#  MIT License was not distributed with this file, you can obtain one
#  at https://opensource.org/licenses/MIT.

from django.conf import settings as django_settings


class _HTCPCPTeaSettings:
    """
    Proxy to the standard Django settings that provides defaults for some
    of this app's settings.
    """

    ALLOW_DEPRECATED_POST = True

    CHECK_FORBIDDEN = True

    RESPONSE_CONTENT_TYPE = None

    DISABLE_CSRF = True

    GET_ADDITIONS = True

    OVERRIDE_ROOT_URI = False

    OVERRIDE_SERVER_NAME = True

    POT_SESSIONS = True

    STRICT_MIME_TYPE = True

    STRICT_REQUEST_BODY = False

    USE_SAFE_HEADER_EXT = True

    def __init__(self, settings_prefix):
        self.prefix = settings_prefix

    def __getattribute__(self, item):
        try:
            return getattr(
                django_settings,
                "{}_{}".format(
                    object.__getattribute__(self, "prefix"),
                    item,
                ),
            )
        except AttributeError:
            return object.__getattribute__(self, item)


htcpcp_settings = _HTCPCPTeaSettings("HTCPCP")



#  Copyright (c) 2019 Brian Schubert
#
#  This file is distributed under the MIT License. If a copy of the
#  MIT License was not distributed with this file, you can obtain one
#  at https://opensource.org/licenses/MIT.

from django.db.models import Q
from django.urls import reverse

from .models import ForbiddenCombination, Pot
from .settings import htcpcp_settings


def build_alternates(index_pot=None):
    """
    Generate the Alternates pairs for available beverages, optionally
    for a specific pot.
    """
    if index_pot:
        pots = (index_pot,)
    else:
        pots = Pot.objects.prefetch_related("supported_teas").all()
    for pot in pots:
        if pot.brew_coffee:
            yield reverse("pot-detail", args=[pot.id]), "message/coffeepot"
        for tea in pot.supported_teas.all():
            yield reverse("pot-detail-tea", args=[pot.id, tea.slug]), "message/teapot"


def render_alternates_header(alternates_pairs):
    """
    Render (uri, content-type) pairs into an RFC 2295 Alternates header value.
    """
    fmt = '{{"{}" {{type {}}}}}'
    return ",".join(fmt.format(*pair) for pair in alternates_pairs)


def resolve_requested_additions(request):
    """
    Return the requested additions for the provided request.

    Additions may be requested in the ``Accept-Additions`` header field, or
    (if the ``HTCPCP_GET_ADDITIONS`` settings is enabled) in the query string
    of a uri.

    Note that the returned additions are not guaranteed to be valid additions
    that are supported by any pot.
    """
    try:
        header = request.META["HTTP_ACCEPT_ADDITIONS"]
        additions = [addition.strip() for addition in header.split(",")]
    except KeyError:
        additions = []

    if htcpcp_settings.GET_ADDITIONS:
        additions += request.GET.dict().keys()

    return additions


def find_forbidden_combinations(requested_additions, tea_slug=None):
    """
    Return the list of ForbiddenCombinations that prohibit some part of the
    requested additions.
    """

    requested_additions = set(requested_additions)

    # Calls to ForbiddenCombination.forbids_additions will need the full
    # list of forbidden additions for each retrieved objects.
    forbidden = ForbiddenCombination.objects.prefetch_related("additions")

    if tea_slug:
        forbidden = forbidden.filter(Q(tea__slug=tea_slug) | Q(tea__isnull=True))
    else:
        forbidden = forbidden.filter(tea__isnull=True)

    # Filter ForbiddenCombinations by what additions they forbid in Python
    # since I could not find a way to accomplish this purely in the database.
    return [fc for fc in forbidden if fc.forbids_additions(requested_additions)]



#  Copyright (c) 2019 Brian Schubert
#
#  This file is distributed under the MIT License. If a copy of the
#  MIT License was not distributed with this file, you can obtain one
#  at https://opensource.org/licenses/MIT.

from .settings import htcpcp_settings
from .utils import render_alternates_header
from .views import brew_pot


class HTCPCPTeaMiddleware:
    HTCPCP_MESSAGE_KEYWORDS = (b"start", b"stop")

    HTCPCP_MIME_TYPES = ("message/teapot", "message/coffeepot")

    def __init__(self, get_response):
        self.get_response = get_response
        self.valid_methods = ("BREW", "WHEN")
        if htcpcp_settings.ALLOW_DEPRECATED_POST:
            self.valid_methods += ("POST",)

    def __call__(self, request):
        htcpcp_valid = True

        if request.method not in self.valid_methods:
            htcpcp_valid = False

        # Resolve HTCPCP message type (start or stop)
        if htcpcp_settings.STRICT_REQUEST_BODY:
            if request.body not in self.HTCPCP_MESSAGE_KEYWORDS:
                htcpcp_valid = False
            else:
                request.htcpcp_message_type = request.body.decode(encoding="utf-8")
        else:
            for keyword in self.HTCPCP_MESSAGE_KEYWORDS:
                if keyword in request.body:
                    request.htcpcp_message_type = keyword.decode(encoding="utf-8")
                    break  # Trigger else branch if no keyword is found
            else:
                htcpcp_valid = False

        if (
            htcpcp_settings.STRICT_MIME_TYPE
            and request.content_type not in self.HTCPCP_MIME_TYPES
        ):
            htcpcp_valid = False

        request.htcpcp_valid = htcpcp_valid

        if (
            htcpcp_valid
            and request.path == "/"
            and htcpcp_settings.OVERRIDE_ROOT_URI
            and htcpcp_settings.STRICT_MIME_TYPE
        ):
            response = brew_pot(request)
        else:
            response = self.get_response(request)

        try:
            alternates_pairs = response.htcpcp_alternates
            response["Alternates"] = render_alternates_header(alternates_pairs)
        except AttributeError:
            pass

        update_server_name = htcpcp_settings.OVERRIDE_SERVER_NAME
        if htcpcp_valid and update_server_name:
            if update_server_name is True:
                server = request.META.get("SERVER_SOFTWARE")
                response["Server"] = "HTCPCP-TEA " + (server if server else "Python")
            elif callable(update_server_name):
                response["Server"] = update_server_name(request, response)
            else:
                response["Server"] = update_server_name.format(**request.META)

        content_type_override = htcpcp_settings.RESPONSE_CONTENT_TYPE
        if htcpcp_valid and content_type_override is not None:
            response["Content-Type"] = content_type_override

        return response



#  Copyright (c) 2019 Brian Schubert
#
#  This file is distributed under the MIT License. If a copy of the
#  MIT License was not distributed with this file, you can obtain one
#  at https://opensource.org/licenses/MIT.
from functools import wraps

from django.http import Http404

from .settings import htcpcp_settings


def require_htcpcp(func):
    """Decorator to make a view only respond to valid HTCPCP requests."""

    @wraps(func)
    def _require_htcpcp(request, *args, **kwargs):
        if not request.htcpcp_valid:
            raise Http404
        return func(request, *args, **kwargs)

    return _require_htcpcp


def safe_condition(token):
    """
    Decorator for adding a ``Safe`` header to a view's response to indicate
    client conditions for the safe handling of a device that brews coffee.

    The decorator will only have effect if the HTCPCP Safe header extension
    option is enabled.

    See `RFC 2324 section 2.2.1.1`_ for further details.

    .. _RFC 2324 section 2.2.1.1: https://tools.ietf.org/html/rfc2324#section-2.2.1.1
    """

    def decorator(func):
        @wraps(func)
        def _wrapped_view(request, *args, **kwargs):
            response = func(request, *args, **kwargs)
            if htcpcp_settings.USE_SAFE_HEADER_EXT:
                response["Safe"] = "if-{}".format(token)
            return response

        return _wrapped_view

    return decorator


safe_require_user_awake = safe_condition("user-awake")
"""
Decorator for adding an "if-user-awake" safe condition to the Safe header
field.

See `RFC 2324 section 2.2.1.1`_ for further details.

.. _RFC 2324 section 2.2.1.1: https://tools.ietf.org/html/rfc2324#section-2.2.1.1
"""



# Generated by Django 2.2.2 on 2019-07-04 11:45

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_htcpcp_tea', '0004_addition_type'),
    ]

    operations = [
        migrations.CreateModel(
            name='ForbiddenCombination',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reason', models.CharField(max_length=180)),
                ('additions', models.ManyToManyField(related_name='forbidden_combinations', to='django_htcpcp_tea.Addition')),
                ('tea', models.ForeignKey(blank=True, help_text='The type of tea that this forbidden combination applies to. Leave blank to apply to all beverages.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='forbidden_combinations', to='django_htcpcp_tea.TeaType')),
            ],
        ),
    ]



#  Copyright (c) 2019 Brian Schubert
#
#  This file is distributed under the MIT License. If a copy of the
#  MIT License was not distributed with this file, you can obtain one
#  at https://opensource.org/licenses/MIT.

from django.contrib import admin
from django.core.exceptions import ImproperlyConfigured
from django.db import models

from .models import Addition, ForbiddenCombination, Pot, TeaType


class RelatedItemsExistsListFilter(admin.SimpleListFilter):
    """Admin list filter for whether an object has a ManyToMany related item."""

    related_item_field = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if not self.related_item_field:
            error_msg = "The list filter {} does not specify a related_item_field"
            raise ImproperlyConfigured(error_msg.format(self.__class__.__name__))

    def lookups(self, request, model_admin):
        return (
            ("y", "Yes"),
            ("n", "No"),
        )

    def queryset(self, request, queryset):
        value = self.value()
        lookup_param = "{}__isnull".format(self.related_item_field)

        if value == "y":
            return queryset.filter(**{lookup_param: False})

        if value == "n":
            return queryset.filter(**{lookup_param: True})


class BrewTeaListFilter(RelatedItemsExistsListFilter):
    """Admin list filter for whether a pots serves any teas."""

    title = "able to brew tea"

    parameter_name = "brew_tea"

    related_item_field = "supported_teas"


class ServedByAPotListFilter(RelatedItemsExistsListFilter):
    """Admin list filter for whether an object is served by a pot."""

    title = "served by a pot"

    parameter_name = "is_served"

    related_item_field = "pot_list"


class SomeCombinationsForbiddenListFilter(RelatedItemsExistsListFilter):
    title = "has forbidden combinations"

    parameter_name = "has_restrictions"

    related_item_field = "forbidden_combinations"


@admin.register(Pot)
class PotAdmin(admin.ModelAdmin):
    search_fields = ("supported_teas__name", "supported_additions__name")

    fields = (("name", "brew_coffee"), "supported_teas", "supported_additions")

    list_display = (
        "id",
        "name",
        "brew_coffee",
        "tea_capable_view",
        "tea_count_view",
        "addition_count_view",
    )

    list_display_links = ("id", "name")

    filter_horizontal = ("supported_teas", "supported_additions")

    list_filter = (
        "brew_coffee",
        BrewTeaListFilter,
        ("supported_teas", admin.RelatedOnlyFieldListFilter),
    )

    save_as = True

    def tea_capable_view(self, obj):
        """Display whether the given pot can brew tea."""
        return obj.tea_capable

    tea_capable_view.boolean = True
    tea_capable_view.short_description = "able to brew tea"
    tea_capable_view.admin_order_field = "supported_teas"

    def tea_count_view(self, obj):
        "Display the number of tea types that the given pot supports."
        return obj.tea_count

    tea_count_view.admin_order_field = "tea_count"
    tea_count_view.short_description = "supported teas"

    def addition_count_view(self, obj):
        "Display the number of addition types that the given pot supports."
        return obj.addition_count

    addition_count_view.admin_order_field = "addition_count"
    addition_count_view.short_description = "supported additions"

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        queryset = queryset.prefetch_related("supported_teas")
        queryset = queryset.prefetch_related("supported_additions")
        return queryset.with_tea_count().with_addition_count()


class PotsServingMixin:
    """
    Mixin to add a 'pots serving' item to a model admin's list_display
    and list_filter
    """

    def get_list_display(self, request):
        fields = super().get_list_display(request)
        serving_callable = self.pots_serving_count_view.__name__
        if serving_callable not in fields:
            return fields + (serving_callable,)
        return fields

    def get_list_filter(self, request):
        return (ServedByAPotListFilter,) + super().get_list_filter(request)

    def pots_serving_count_view(self, obj):
        """Display the number of pots that serve the given object."""
        return obj.pot_count

    pots_serving_count_view.admin_order_field = "pot_count"
    pots_serving_count_view.short_description = "pots serving"

    def get_queryset(self, request):
        return (
            super()
            .get_queryset(request)
            .annotate(pot_count=models.Count("pot_list", distinct=True))
        )


class HasForbiddenCombinationsMixin:
    """
    Mixin to add a 'forbidden combinations' item to a model admin's
    list_display and list_filter.
    """

    def get_list_display(self, request):
        fields = super().get_list_display(request)
        forbidden_callable = self.forbidden_combination_count_view.__name__
        if forbidden_callable not in fields:
            return fields + (forbidden_callable,)
        return fields

    def get_list_filter(self, request):
        return super().get_list_filter(request) + (SomeCombinationsForbiddenListFilter,)

    def forbidden_combination_count_view(self, obj):
        """
        Display the number of forbidden combinations that include the
        given object.
        """
        return obj.forbidden_count

    forbidden_combination_count_view.admin_order_field = "forbidden_count"
    forbidden_combination_count_view.short_description = "forbidden combinations"

    def get_queryset(self, request):
        return (
            super()
            .get_queryset(request)
            .annotate(
                forbidden_count=models.Count("forbidden_combinations", distinct=True)
            )
        )


class ForbiddenCombinationInline(admin.TabularInline):
    model = ForbiddenCombination

    fields = ("reason", "additions")

    extra = 0

    filter_horizontal = ("additions",)

    classes = ("collapse",)

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        return queryset.select_related("tea").prefetch_related("additions")


@admin.register(TeaType)
class TeaTypeAdmin(PotsServingMixin, HasForbiddenCombinationsMixin, admin.ModelAdmin):
    inlines = (ForbiddenCombinationInline,)

    search_fields = ("name",)

    prepopulated_fields = {"slug": ("name",)}

    fieldsets = (
        (None, {"fields": ("name",)}),
        (
            "Advanced",
            {
                "classes": ("collapse",),
                "fields": ("slug",),
            },
        ),
    )


@admin.register(Addition)
class AdditionAdmin(PotsServingMixin, HasForbiddenCombinationsMixin, admin.ModelAdmin):
    search_fields = ("name",)

    list_display = ("name", "type")

    list_filter = ("type",)

    radio_fields = {"type": admin.HORIZONTAL}


@admin.register(ForbiddenCombination)
class ForbiddenCombinationAdmin(admin.ModelAdmin):
    list_display = ("__str__", "reason")

    search_fields = ("additions__name", "tea__name")

    list_filter = (
        ("tea", admin.RelatedOnlyFieldListFilter),
        ("additions", admin.RelatedOnlyFieldListFilter),
    )

    filter_horizontal = ("additions",)

    save_as = True

    formfield_overrides = {
        models.ForeignKey: {"empty_label": "------ All ------"},
    }

    def get_queryset(self, request):
        queryset = super().get_queryset(request)
        return queryset.select_related("tea").prefetch_related("additions")



# Generated by Django 2.2.2 on 2019-06-17 19:39

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Addition',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=35, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='TeaType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=35, unique=True, verbose_name='Tea name')),
                ('slug', models.SlugField(unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='Pot',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=35, unique=True)),
                ('brew_coffee', models.BooleanField(default=True, verbose_name='Able to brew coffee?')),
                ('supported_additions', models.ManyToManyField(to='django_htcpcp_tea.Addition')),
                ('supported_teas', models.ManyToManyField(to='django_htcpcp_tea.TeaType')),
            ],
        ),
    ]



#  Copyright (c) 2019 Brian Schubert
#
#  This file is distributed under the MIT License. If a copy of the
#  MIT License was not distributed with this file, you can obtain one
#  at https://opensource.org/licenses/MIT.

from datetime import datetime

from django.shortcuts import get_object_or_404, render

from .decorators import require_htcpcp
from .models import Addition, Pot
from .settings import htcpcp_settings
from .utils import (
    build_alternates,
    find_forbidden_combinations,
    resolve_requested_additions,
)


@require_htcpcp
def brew_pot(request, pot_designator=None, tea_type=None):
    if not pot_designator:
        alternates = list(build_alternates())
        context = {"alternatives": alternates}
        response = render(
            request, "django_htcpcp_tea/options.html", context, status=300
        )
        response.htcpcp_alternates = alternates
        return response

    if request.method == "WHEN" and request.htcpcp_message_type == "start":
        return render(
            request,
            "django_htcpcp_tea/400.html",
            {"error_reason": "Cannot start a beverage with a WHEN request."},
            status=400,
        )

    pot = get_object_or_404(Pot, id=pot_designator)

    if _request_for_tea(request, tea_type):
        response = _precheck_teapot(request, pot, tea_type)
        # Beverage name only required when starting a new beverage
        beverage_name = "{} Tea".format(tea_type.capitalize) if tea_type else None
    else:
        response = _precheck_coffee(request, pot)
        beverage_name = "coffee"

    if response is None:
        addition_names = resolve_requested_additions(request)
        try:
            additions = list(pot.fetch_additions(addition_names))
        except Addition.DoesNotExist:
            # Fetch all supported additions for the requested pot.
            # Note that this will result in an additional query.
            context = {"supported_additions": pot.supported_additions.all()}
            return render(request, "django_htcpcp_tea/406.html", context, status=406)

        if htcpcp_settings.CHECK_FORBIDDEN:
            forbidden = find_forbidden_combinations(additions, tea_type)

            if forbidden:
                context = {"matched_combinations": forbidden}
                return render(
                    request, "django_htcpcp_tea/403.html", context, status=403
                )

        if htcpcp_settings.POT_SESSIONS:
            response = _finalize_beverage_with_session(
                request, pot, beverage_name, additions
            )
        else:
            response = _finalize_beverage(request, pot, beverage_name, additions)

    return response


def _request_for_tea(request, tea_type):
    """
    Determine whether the given request is for tea.

    If the ``STRICT_MIME_TYPE`` settings is enabled, then the MIME type of the
    request is used to unambiguously categorize it as a tea or coffee request.

    If strict MIME types are not enforced, the requested beverage type is
    inferred from the existence of a tea type in the request URI and the HTCPCP
    MIME type, if one is provided.
    """
    if request.content_type == "message/teapot":
        return True
    elif not htcpcp_settings.STRICT_MIME_TYPE:
        return tea_type
    else:
        return False


def _precheck_coffee(request, pot):
    """
    Return a response if a precondition for coffee requests is not satisfied,
    else None.
    """
    if pot.is_teapot:
        return render(request, "django_htcpcp_tea/418.html", status=418)

    if not pot.brew_coffee:
        return render(
            request,
            "django_htcpcp_tea/503.html",
            {"error_reason": "Pot out of service. No coffee or tea available."},
            status=503,
        )

    return None


def _precheck_teapot(request, pot, tea):
    """
    Return a response if a precondition for tea requests is not satisfied, else
    None.
    """
    if request.htcpcp_message_type == "start":
        if not tea:  # Require tea type only when starting a new beverage
            alternatives = list(build_alternates(index_pot=pot))
            context = {"alternatives": alternatives}
            response = render(
                request, "django_htcpcp_tea/options.html", context, status=300
            )
            response.htcpcp_alternates = alternatives
            return response
        elif tea not in pot.supported_teas.values_list("slug", flat=True):
            return render(
                request,
                "django_htcpcp_tea/503.html",
                {
                    "error_reason": "{} is not available for this pot".format(
                        tea.capitalize
                    )
                },
                status=503,
            )
    return None


def _finalize_beverage(request, pot, beverage_name, additions):
    """
    Return a response to the beverage request according to the HTCPCP standard
    without referencing the user's session.
    """
    context = {
        "pot": pot,
        "beverage": beverage_name,
        "additions": additions,
    }

    if request.htcpcp_message_type == "start":
        if beverage_name == "coffee":
            # Display alternatives when brewing coffee per RFC 7168 section 2.1.1
            alternates = list(build_alternates())
            context["alternatives"] = alternates
            response = render(
                request, "django_htcpcp_tea/brewing.html", context, status=202
            )  # Accepted
            response.htcpcp_alternates = alternates
        else:
            response = render(
                request, "django_htcpcp_tea/brewing.html", context, status=202
            )  # Accepted
    else:  # request.htcpcp_message_type == 'stop':
        if any(addition.is_milk for addition in additions):
            response = render(
                request, "django_htcpcp_tea/pouring.html", context, status=200
            )  # Ok
        else:
            response = render(
                request, "django_htcpcp_tea/finished.html", context, status=201
            )  # Created

    return response


def _finalize_beverage_with_session(request, pot, beverage_name, additions):
    """
    Return a response to the beverage request according to the HTCPCP standard
    by referencing the current state of the user's session.
    """
    session_key = "htcpcp_pot_{}".format(pot.id)
    pot_status = request.session.get(session_key)

    context = {
        "pot": pot,
        "beverage": beverage_name,
        "additions": additions,
    }

    # TODO add brew time and additions display to finished template

    if pot_status:
        # 'stop' requests may not be able to reproduce the name of a beverage
        # or the additions that were requested, so override these values in the
        # context with the ones stored in the user's session.
        context["beverage"] = pot_status["beverage"]
        context["additions"] = pot_status["additions"]

        if request.htcpcp_message_type == "start":
            context["error_reason"] = "Pot is busy and cannot start a new beverage."
            response = render(
                request, "django_htcpcp_tea/503.html", context, status=503
            )
        else:  # htcpcp_message_type == 'stop'
            if request.method == "WHEN":
                if pot_status["currently_pouring"]:
                    response = render(
                        request, "django_htcpcp_tea/finished.html", context, status=201
                    )
                    del request.session[session_key]
                else:
                    context[
                        "error_reason"
                    ] = 'No milk is being poured. Please stop shouting "WHEN!"'
                    return render(
                        request, "django_htcpcp_tea/400.html", context, status=400
                    )
            else:
                if pot_status["currently_pouring"]:
                    context[
                        "error_reason"
                    ] = 'Milk is currently being poured. Please say "WHEN"'
                    response = render(
                        request, "django_htcpcp_tea/400.html", context, status=400
                    )
                elif pot_status["needs_milk"]:  # Stop brewing and begin pouring milk
                    response = render(
                        request, "django_htcpcp_tea/pouring.html", context, status=200
                    )
                    request.session[session_key].update(
                        {
                            "needs_milk": False,
                            "currently_pouring": True,
                        }
                    )
                    request.session.modified = True
                else:  # Stop brewing. No milk required.
                    response = render(
                        request, "django_htcpcp_tea/finished.html", context, status=201
                    )
                    del request.session[session_key]
    elif request.htcpcp_message_type == "start":
        # New session, and the client requested a new beverage
        if beverage_name == "coffee":
            # Display alternatives when brewing coffee per RFC 7168 section 2.1.1
            context["alternatives"] = build_alternates(index_pot=pot)
        response = render(
            request, "django_htcpcp_tea/brewing.html", context, status=202
        )  # Accepted
        request.session[session_key] = {
            "beverage": beverage_name,
            # Serialize the requested additions to as dictionaries for storage
            # in the user's session since we do not need actual Addition objects
            # to display the additions during future requests.
            "additions": [
                {"name": a.name, "get_type_display": a.get_type_display()}
                for a in additions
            ],
            "needs_milk": any(addition.is_milk for addition in additions),
            "currently_pouring": False,
            "start_time": datetime.utcnow().timestamp(),
        }
    else:
        reason = (
            "No beverage is being brewed by this pot, but the "
            "request did not indicate that a new beverage should be "
            "brewed"
        )
        response = render(
            request, "django_htcpcp_tea/400.html", {"error_reason": reason}, status=400
        )

    return response


if htcpcp_settings.DISABLE_CSRF:
    # Mark the HTCPCP view function as being exempt from the CSRF view
    # protection. This is the same as using the csrf_exempt decorator
    # from django.views.decorators.csrf.
    brew_pot.csrf_exempt = True



#  Copyright (c) 2019 Brian Schubert
#
#  This file is distributed under the MIT License. If a copy of the
#  MIT License was not distributed with this file, you can obtain one
#  at https://opensource.org/licenses/MIT.

__version__ = "0.8.1"

default_app_config = "django_htcpcp_tea.apps.HTCPCPTeaConfig"
